/*
**  WASABI-Hot! version 1.2b   
**
**
**      -- copyright (c) 2001-2003 by Philips Japan, Ltd. -- All rights reserved --
**
**
**      ** This code has been made to check/learn                          ** 
**      **                             the ISP1362/ISP1363 functionalities **
**      ** Release 08-Dec-2003                                             **
**
**      OKANO, Akifumi
**      
**		Application Laboratory, Portable Systems
**      Semiconductors Div, Philips Japan Ltd.
**      akifumi.okano@philips.com
**      +81-3-3740-4668
*/

//	12-Nov.-2001 fseek() is removed to reduce standard library call.
//	12-Nov.-2001 File access changed from fread() to read()
//	13-Nov.-2001 Bug fixed
//	28-Jan.-2002 Simultaneous printing support
//	30-Jan.-2001  Source code directory structure changed


/****************************************************************************/
/*	includes																*/
/****************************************************************************/

#include		<stdio.h>
#include		<stdlib.h>
#include		<alloc.h>
#include		<string.h>
#include		<ctype.h>
#include		<fcntl.h>
#include		<dos.h>

#include		"_hc_core/atl_mix.h"
#include		"_hc_core/dev_ep.h"
#include		"_hc_core/transfer.h"

#include		"_hc_cls/cls_hndl.h"

#include		"class_dr/audio/sing.h"
#include		"class_dr/printer/printer.h"

#include		"ui.h"
#include		"general.h"


/****************************************************************************/
/*	constants																*/
/****************************************************************************/

#define			PRINT_BUFFER_SIZE			(10 * (1 << 10))	//	Bytes
#define			PRINT_BUFFER_FILL_REQUEST	PRINT_BUFFER_SIZE

#define			DEFAULT_PRINT_FILE			"default.pcl"

#define			NUM_OF_VENDER				3
#define			PAGE_END_CHARS				3

#define			NAK_RETRY_WAIT				20		//	ms


/****************************************************************************/
/*	global vars																*/
/****************************************************************************/

char					gp_printer_default_file_path[ FILE_NAME_STR_LENGTH ]		= "";

unsigned char			g_init_command_array[ 2 ][ 8 ]							=  {
																						{ 0x01, 0x0B, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00 },
																						{ 0xA1, 0x00, 0x00, 0x00, 0x00, 0x00, 0xA1, 0x00 }
																					};


ptr_vid_suffix			printer_vid_suffix[ NUM_OF_VENDER ]						= {
																						{	VENDER_ID_HP,    DEFAULT_FILE_SUFFIX_HP    },
																						{	VENDER_ID_EPSON, DEFAULT_FILE_SUFFIX_EPSON },
																						{	VENDER_ID_CANON, DEFAULT_FILE_SUFFIX_CANON }
																					};
char					text_page_end[ PAGE_END_CHARS ]							= { 0x0D, 0x0A, 0x0C };

unsigned short			num_of_text_page_end_chars								= PAGE_END_CHARS;			



unsigned char			g_num_of_printer_devices								= 0;
unsigned char			g_printer_device_address_list[ MAX_PRINTER_DEVICES ]	= { 0, 0 };
																		

/****************************************************************************/
/*	function prototypes														*/
/****************************************************************************/

void				printer_main( void );
printer_instance	*printer_job_initialization( printer_instance *pi_ptr );
void				start_printer( printer_instance *pi_ptr );
void				stop_printer( printer_instance *pi_ptr );
void				read_file_and_bulk_out( endpoint_info_ptr epi_ptr );
void				async_read_file_and_bulk_out( void );
void				printer_maintenance( printer_instance	*pi_ptr );

void				printer_progress_indicator( void );
unsigned long		bulk_transfer_service_from_file( unsigned char *buffer, int g_printer_image_file_handle, unsigned long file_size, endpoint_info *epi_ptr );
int					print_file_name_input_dialog( char *file_name, printer_instance *pi_ptr );
char				*vender_independent_suffix( unsigned char address );
char				isThisTextFile( char *file_name );
unsigned long 		printer_file_size( int g_printer_image_file_handle );
void				printer_status_monitor( unsigned char monitor_index );
unsigned char		is_NAK_wait( printer_instance *pi_ptr );
unsigned char		number_of_current_printing_job( void );

unsigned long		file_size( int fh );

printer_instance	*select_a_target_printer( void );

/****************************************************************************/
/*	function definitions													*/
/****************************************************************************/


/*********															*********/
/*																			*/
/*	Generic functions														*/
/*																			*/
/*		These functions are called from application							*/
/*																			*/
/*********															*********/


void printer_main( void )
{
	printer_instance *pi_ptr;
	
	mprintf( LIGHTGREEN, CONTINUE, "\r\n\r\n  PrinterTest started\r\n" );

	if ( NULL == (pi_ptr	= select_a_target_printer()) )				//	select_a_target_printer() is defined in ui.c
	{
		mprintf( YELLOW, CONTINUE, "Print cancelled.\r\n" );
		return;
	}
	
	if ( NULL != pi_ptr->image_buffer_ptr )
	{
		stop_printer( pi_ptr );
		mprintf( YELLOW, CONTINUE, "    Printing terminated by user.\r\n" );
		
		return;
	}
		
	if ( NULL == (pi_ptr	= printer_job_initialization( pi_ptr )) )
		return;
		
	start_printer( pi_ptr );
}


printer_instance *printer_job_initialization( printer_instance *pi_ptr )
{
	device_instance		*dvi_ptr;
	char				file_name[ FILE_NAME_STR_LENGTH ];
	double				size;
	unsigned char		i;
#ifdef TEXT_FILE_PRINTING
	char				text_mode;
#endif


	/*								*/
	/*  Selection for printer file	*/
	/*								*/
	
	print_file_name_input_dialog( file_name, pi_ptr );

	if ( -1 == *file_name )
	{
		mprintf( LIGHTGREEN, CONTINUE, "    Pirnt canceled by user by pressing [Esc] key.\r\n" );
		return ( NULL );
	}
	
	if ( !(*file_name) )
		strcpy( file_name, DEFAULT_PRINT_FILE );
	
	/*									*/
	/*  Check TEXT file					*/
	/*									*/

#ifdef TEXT_FILE_PRINTING
	if ( -1 == (text_mode	= isThisTextFile( file_name )) )
	{
		mprintf( LIGHTGREEN, CONTINUE, "    error @ opening print source file \"%s\"\r\n", file_name );
		return ( NULL );
	}
#endif
	
	/*									*/
	/*  File open 						*/
	/*									*/
	
	if ( 0 != (_dos_open( file_name, O_RDONLY, &( pi_ptr->image_file_handle) )) )
	{
		mprintf( LIGHTGREEN, CONTINUE, "    error @ opening print source file \"%s\"\r\n", file_name );
		return ( NULL );
	}
	
	pi_ptr->file_size	= printer_file_size(  pi_ptr->image_file_handle );
	
	mprintf( LIGHTGREEN, CONTINUE, "\r\n" );
	mprintf( LIGHTGREEN, CONTINUE, "    printing file \"%s\"\r\n", file_name );
	mprintf( LIGHTGREEN, CONTINUE, "    printing data size %lu ",  pi_ptr->file_size );
	
	if (  pi_ptr->file_size >> 20 )
		mprintf( LIGHTGREEN,  CONTINUE, "(%.2fM)", (double) pi_ptr->file_size / (double)(1L << 20) );
	else
		mprintf( LIGHTGREEN,  CONTINUE, "(%.2fK)", (double) pi_ptr->file_size / (double)(1 << 10) );
		
	mprintf( LIGHTGREEN,  CONTINUE, " bytes.\r\n" );
		
	
	mprintf(      GREEN, CONTINUE, "    printing can be aborted by key [p].\r\n" );
	mprintf(      GREEN, CONTINUE, "    other key press will be ignored.\r\n" );
	
		
	/*									*/
	/*  Allocating application buffer	*/
	/*									*/
	
	if ( NULL == ( pi_ptr->image_buffer_ptr	= (unsigned char *)malloc( (PRINT_BUFFER_SIZE & ~0x3F) * sizeof( unsigned char ) )) )	//	buffer size must be "N * 64 bytes"
		mprintf( LIGHTRED, ABORT, "malloc error @ printer_main\r\n" );
	
	/*									*/
	/*  Set target endpoint				*/
	/*									*/
	
	dvi_ptr		= devep_find_device( pi_ptr->target_device_address );

	for ( i = 1; i <= 2; i++ )
		if ( NULL != (pi_ptr->endpoint_ptr	= (dvi_ptr->epi_ptr)[ EpOUT ][ i ]) )
			break;

	return ( pi_ptr );
}


void test_func( void )
{
	device_instance		*dvi_ptr;
	unsigned char		i;

	for ( i = 0; i < MAX_PRINTER_DEVICES; i++ )
		mprintf( WHITE, CONTINUE, "    addr:%u = 0x%08lX", g_printer_device_address_list[ i ], ((printer_instance *)(devep_find_device( g_printer_device_address_list[ i ] )->class_instance_ptr))->time_for_action );

	mprintf( WHITE, CONTINUE, "\r" );
}




void start_printer( printer_instance *pi_ptr )
{
	unsigned short		size;

	if ( (NO_OPEN_ATL_TRANSFER_SLOT == (pi_ptr->tr_index	= atlmix_get_open_transfer_index( BULK_TRANSFER ))) )
		mprintf( LIGHTRED, ABORT, "error @ transfer_control_transaction, transfer list full.\r\n" );

	pi_ptr->transferred_size		= 0L;
	pi_ptr->time_for_action			= ACTION_START;
	
	//	re-install monitor line to clear the display
	ui_install_status_monitor_custom_routine( PRINTER_STATUS_MONITOR_LINE + pi_ptr->pi_index, printer_status_monitor );

	if ( 1 == number_of_current_printing_job() )
		gene_install_asynchronous_periodic_process( 1, async_read_file_and_bulk_out );

//gene_install_asynchronous_periodic_process( 9, test_func );

	//	Start print by setting buffer fill request flag;
	pi_ptr->buffer_status	= PRINT_BUFFER_SIZE;
}


void stop_printer( printer_instance *pi_ptr )
{
	pi_ptr->time_for_action				= ACTION_IDLE;

	atlmix_free_transfer_index( BULK_TRANSFER, pi_ptr->tr_index );
	
	free( pi_ptr->image_buffer_ptr );
	pi_ptr->image_buffer_ptr			= NULL;
	pi_ptr->buffer_status				= 0;
	
	_dos_close( pi_ptr->image_file_handle );
	
	if ( 0 == number_of_current_printing_job() )
		gene_install_asynchronous_periodic_process( 1, NULL );

	mprintf( LIGHTGREEN, CONTINUE, "\r\n    printer operation finished.\r\n" );
	mprintf(      GREEN, CONTINUE,     "    %lu", pi_ptr->transferred_size );
	
	if ( pi_ptr->transferred_size >> 20 )
		mprintf( GREEN,  CONTINUE, " (%.2fM)", (double)pi_ptr->transferred_size / (double)(1L << 20) );
	else
		mprintf( GREEN,  CONTINUE, " (%.2fK)", (double)pi_ptr->transferred_size / (double)(1 << 10) );

	mprintf( GREEN, CONTINUE, " bytes transferred.\r\n\r\n" );

	//	re-install monitor line to clear the display
	ui_install_status_monitor_custom_routine( PRINTER_STATUS_MONITOR_LINE + pi_ptr->pi_index, printer_status_monitor );
}




/*********															*********/
/*																			*/
/*	Printer job maintenance function										*/
/*																			*/
/*		This functions is called from ATL scheduller as callback function	*/
/*																			*/
/*********															*********/

void async_read_file_and_bulk_out( void )
{
	unsigned short		size;
	unsigned char		read_flag;
	printer_instance	*pi_ptr;
	unsigned char		i;

	for ( i = 0; i < MAX_PRINTER_DEVICES; i++ )
	{
		if ( g_printer_device_address_list[ i ] )
		{
			pi_ptr	= (printer_instance *)(devep_find_device( g_printer_device_address_list[ i ] )->class_instance_ptr);

			if ( pi_ptr->time_for_action < gp_sof_counter )
				printer_maintenance( pi_ptr );
		}
	}
}


void printer_maintenance( printer_instance	*pi_ptr )
{
	unsigned short		transferred_size;
	unsigned short		size;

	if ( pi_ptr->time_for_action == ACTION_START )
	{
		transferred_size					= 0;
		pi_ptr->requested_transfer_size		= 0;
	}
	else
	{
		transferred_size	= (pi_ptr->tr_instance_ptr)->transferred_size;

		if ( (transferred_size == 0) && (pi_ptr->time_for_action == ACTION_IMMIDIATE) )
		{
			pi_ptr->time_for_action		= gp_sof_counter + NAK_RETRY_WAIT;
			return;
		}
	}
	
	pi_ptr->time_for_action		 = ACTION_IDLE;

	pi_ptr->transferred_size	+= transferred_size;	//	Update total transferred size
	pi_ptr->buffer_status		+= transferred_size;	//	Transferred size from previous buffer filling
	
	size						 = pi_ptr->requested_transfer_size - transferred_size;

	if ( size == 0 )	//	Needs file read to fill buffer
	{
		pi_ptr->buffer_status				= 0;

		_dos_read( pi_ptr->image_file_handle, pi_ptr->image_buffer_ptr, PRINT_BUFFER_SIZE, (unsigned *)(&size) );

		if ( size )
			pi_ptr->tr_instance_ptr		= atlmix_set_transfer( BULK_TRANSFER, 0, pi_ptr->tr_index, pi_ptr->image_buffer_ptr, size, OUT, pi_ptr->endpoint_ptr, read_file_and_bulk_out );
		else
			stop_printer( pi_ptr );
			
		pi_ptr->requested_transfer_size		= size;
	}
	else
	{
		pi_ptr->requested_transfer_size		= size;

		atlmix_set_transfer( BULK_TRANSFER, 0, pi_ptr->tr_index, (pi_ptr->image_buffer_ptr) + pi_ptr->buffer_status, size, OUT, pi_ptr->endpoint_ptr, read_file_and_bulk_out );
	}
}


void read_file_and_bulk_out( endpoint_info_ptr epi_ptr )
{
	printer_instance	*pi_ptr;

	pi_ptr						= (printer_instance *)(devep_find_device( epi_ptr->dev_addr )->class_instance_ptr);
	pi_ptr->time_for_action		= ACTION_IMMIDIATE;
}


/*********															*********/
/*																			*/
/*	Device class handling functions											*/
/*																			*/
/*		This functions is called from functions in cls_hndl.c as callback	*/
/*		functions. 															*/
/*		All these functions are needed to be installed by 					*/
/*		"clshndl_initialization_method_install()"	in "cls_hndl.c".			*/
/*																			*/
/*		These functions will be called at printer device attach/detach		*/
/*		events. 															*/
/*																			*/
/*********															*********/


unsigned short printer_init_commands( device_instance *dvi_ptr )
{
	unsigned short			err;
	unsigned char			*buff_ptr;
	printer_instance		*pi_ptr;
	int						c_pos;
	unsigned char			list_index;

	if ( g_num_of_printer_devices == MAX_PRINTER_DEVICES )
	{
		mprintf( LIGHTRED, CONTINUE, "Too many printers. This code supprt only %d printer devices.", MAX_PRINTER_DEVICES );
		return ( 1 );
	}
	
	if ( devep_regist_config_descriptors( dvi_ptr, 1, 0 ) )
		return ( error );	

	g_num_of_printer_devices++;
	
	for ( list_index = 0; list_index < MAX_PRINTER_DEVICES; list_index++ )
	{
		if ( g_printer_device_address_list[ list_index ] == 0 )
		{
			g_printer_device_address_list[ list_index ]	= dvi_ptr->address;
			break;
		}
	}

	/*
	**	Make a printer instance for printer device
	*/

	if ( NULL == (pi_ptr		= (printer_instance *)malloc( sizeof( printer_instance ) )) )
		mprintf( LIGHTRED, CONTINUE, "error @ printer_init_commands : making printer class instance.\r\n" );
		
	dvi_ptr->class_instance_ptr		= pi_ptr;
	
	pi_ptr->time_for_action			= ACTION_IDLE;
	pi_ptr->image_buffer_ptr		= NULL;
	pi_ptr->target_device_address	= dvi_ptr->address;
	pi_ptr->buffer_status			= 0;
	
	/*
	**	Setting for monitor status display (variable setting for callback function "printer_status_monitor()")
	*/

	c_pos	= ui_unicode_to_C_string( pi_ptr->monitor_string, (dvi_ptr->string_manufacturer) + 1, (((std_string_descriptor *)(dvi_ptr->string_manufacturer))->bLength) >> 1 );
	ui_unicode_to_C_string( (pi_ptr->monitor_string) + c_pos + 1, (dvi_ptr->string_product) + 1, (((std_string_descriptor *)(dvi_ptr->string_product))->bLength) >> 1 );
	
	*((pi_ptr->monitor_string) + c_pos)		= ' ';
	
	pi_ptr->pi_index			= list_index;
	pi_ptr->operation_flag		= 1;
	
	ui_install_status_monitor_custom_routine( PRINTER_STATUS_MONITOR_LINE + list_index, printer_status_monitor );

	return ( 0 );
}


unsigned short printer_dispose_process( device_instance *dvi_ptr )
{
	printer_instance	*pi_ptr;
	unsigned char		pi_index;

	pi_ptr		= (printer_instance *)(dvi_ptr->class_instance_ptr);

	if ( pi_ptr->image_buffer_ptr != NULL )  //	Check printing process is in process ot not.
		stop_printer( pi_ptr );
				
	pi_ptr->target_device_address	= 0;
	pi_index						= pi_ptr->pi_index;
	
	ui_install_status_monitor_custom_routine( PRINTER_STATUS_MONITOR_LINE + pi_index, NULL );
	
	free( dvi_ptr->class_instance_ptr );
	dvi_ptr->class_instance_ptr		= NULL;
	
	g_printer_device_address_list[ pi_index ]	= 0;
	g_num_of_printer_devices--;

	return ( 0 );
}


/*********															*********/
/*																			*/
/*	Utility functions														*/
/*																			*/
/*********															*********/


int print_file_name_input_dialog( char *file_name, printer_instance *pi_ptr )
{
	mprintf( YELLOW, CONTINUE, "\r\n" );
	mprintf( YELLOW, CONTINUE, "    type in the file name (and path) to print.\r\n" );
	mprintf( YELLOW, CONTINUE, "    just [Return] makes to use default file.\r\n\r\n" );
	
	ui_get_string_from_console( file_name, FILE_NAME_STR_LENGTH, WHITE, "    file name >> ", YELLOW, vender_independent_suffix(  pi_ptr->target_device_address ), gp_printer_default_file_path, NULL );
	mprintf( YELLOW, CONTINUE, "\r\n" );
	
	return ( !(*file_name) );
}


char *vender_independent_suffix( unsigned char address )
{
	int		i;
	
	for ( i = 0; i < NUM_OF_VENDER; i++ )
		if ( venderID( devep_find_device( address ) ) == printer_vid_suffix[ i ].vid )
			return ( printer_vid_suffix[ i ].suffix );
	
	return ( 0 );
}


char isThisTextFile( char *file_name )
{
	//	This function returns 0xFF, if file can not be opened.
	
	FILE	*fp;
	char	data;
	
	if ( NULL == (fp	= fopen( file_name, "rb" )) )
		return ( 0xFF );	

	fread( &data, sizeof( unsigned char ), 1, fp );
	
	fclose( fp );
	
	if ( isprint( data ) )
		return ( 1 );
	else
		return ( 0 );
}


#include	<sys\stat.h>

unsigned long file_size( int fh )
{
	return ( printer_file_size( fh ) );
}


unsigned long printer_file_size( int fh )
{
	struct stat		stat_buffer;
	
	fstat( fh, &stat_buffer );

	return ( (unsigned long)(stat_buffer.st_size) );
}


unsigned char number_of_current_printing_job( void )
{
	unsigned char	i;
	unsigned char	count	= 0;

	for ( i = 0; i < MAX_PRINTER_DEVICES; i++ )
		if ( g_printer_device_address_list[ i ] )
			count	+= (((printer_instance *)(devep_find_device( g_printer_device_address_list[ i ] )->class_instance_ptr))->image_buffer_ptr) ? 1 : 0;

	return ( count );
}



/*********															*********/
/*																			*/
/*	User interface (to show the printer job status) function				*/
/*																			*/
/*		This functions is called from "ui_status_monitor()" in "ui.c" as 		*/
/*		callback function. 													*/
/*		This routine will be installed by "printer_init_commands()"	at		*/
/*		printer device enumeration.											*/
/*																			*/
/*********															*********/

//#define		DEBUG_MONITOR

void printer_status_monitor( unsigned char monitor_index )
{
	device_instance		*dvi_ptr;
	printer_instance	*pi_ptr;
	unsigned char		current_state;
	unsigned char		prev_state;

	monitor_index	-= PRINTER_STATUS_MONITOR_LINE;
	
	if ( !g_printer_device_address_list[ monitor_index ] )
		return;
	
	dvi_ptr			= devep_find_device( g_printer_device_address_list[ monitor_index ] );
	pi_ptr			= (printer_instance *)(dvi_ptr->class_instance_ptr);
	prev_state		= pi_ptr->operation_flag;
	
	current_state	= (pi_ptr->image_buffer_ptr) ? 1 : 0;
	
#ifdef DEBUG_MONITOR

	mprintf( BG_BLUE | WHITE    , CONTINUE, "    Printer" );

	if ( pi_ptr->image_buffer_ptr )
		mprintf( BG_BLUE | LIGHTGRAY, CONTINUE, "  : %p, %lu, %lu, %d", pi_ptr, pi_ptr->file_size, pi_ptr->transferred_size, pi_ptr->operation_flag );
	else
		mprintf( BG_BLUE | LIGHTGRAY, CONTINUE, "  : %p, %p", pi_ptr, pi_ptr->image_buffer_ptr );
		
#else

	if ( current_state && !prev_state )
	{
		mprintf( BG_BLUE | WHITE    , CONTINUE, "    Printer" );
		mprintf( BG_BLUE | LIGHTGRAY, CONTINUE, "(addr %2d) : ",pi_ptr->target_device_address );
		mprintf( BG_BLUE | WHITE    , CONTINUE, "printing " );
		mprintf( BG_BLUE | LIGHTGRAY, CONTINUE, "%s %5.1f%% (%8lu / %8lu bytes)", is_NAK_wait( pi_ptr ) ? "NAKing": "      ", ((double)(pi_ptr->transferred_size) / (double)(pi_ptr->file_size)) * 100.00, pi_ptr->transferred_size, pi_ptr->file_size );
	}
	
	if ( current_state )
	{
		gotoxy( 33, wherey() );
		mprintf( BG_BLUE | LIGHTGRAY, CONTINUE, "%s %5.1f%% (%8lu", is_NAK_wait( pi_ptr ) ? "NAKing": "      ", ((double)(pi_ptr->transferred_size) / (double)(pi_ptr->file_size)) * 100.00, pi_ptr->transferred_size );
	}
	
	if ( prev_state && !current_state )
	{
		mprintf( BG_BLUE | WHITE    , CONTINUE, "    Printer" );
		mprintf( BG_BLUE | LIGHTGRAY, CONTINUE, "(addr %2d) : ready. (%s)", pi_ptr->target_device_address, pi_ptr->monitor_string );
	}
	
	pi_ptr->operation_flag	= current_state;
	
#endif
}


unsigned char is_NAK_wait( printer_instance *pi_ptr )
{
	if ( 
			(pi_ptr->time_for_action == ACTION_IMMIDIATE)
				||
			(pi_ptr->time_for_action == ACTION_START    )
				||
			(pi_ptr->time_for_action == ACTION_IDLE     )
		)
		return ( False );
	else
		return ( True );
}


printer_instance *select_a_target_printer( void )
{
	printer_instance	*pi_list[ MAX_PRINTER_DEVICES ];
	unsigned char		num_of_prn;
	unsigned char		next_address;
	device_instance		*dvi_ptr;
	printer_instance	*pip;
	int					key;
	int					i;

	for ( num_of_prn = 0, next_address = 0; num_of_prn < MAX_PRINTER_DEVICES; num_of_prn++ )
	{
		if ( NULL == (dvi_ptr	= devep_find_class_interface( PRINTER_CLASS_INTERFACE, next_address )) )
			break;
		
		pi_list[ num_of_prn ]	= (printer_instance *)(dvi_ptr->class_instance_ptr);
		next_address			= dvi_ptr->address + 1;
	}

	if ( num_of_prn == 0 )
	{
		mprintf( YELLOW, CONTINUE, "No printer calss device has been found.\r\n" );
		return ( NULL );
	}
	else if ( num_of_prn == 1 )
	{
		return ( pi_list[ 0 ] );
	}	

	mprintf( YELLOW, CONTINUE, "    %d printers are on the USB.\r\n\r\n", num_of_prn );
	mprintf( YELLOW, CONTINUE, "    Choose a printer\r\n" );
	mprintf( YELLOW, CONTINUE, "    ^^^^^^^^^^^^^^^^\r\n" );
	

	for ( i = 0; i < num_of_prn; i++ )
	{
		pip		= pi_list[ i ];

		mprintf( YELLOW, CONTINUE, "      [%d] - Addr %2d ", i, pip->target_device_address );
		mprintf( YELLOW, CONTINUE, "(%s) : ", (pip->image_buffer_ptr) ? "busy" : "idle" );
		
		dvi_ptr		= devep_find_device( pip->target_device_address );
	
		if ( dvi_ptr->string_manufacturer != NULL )
			ui_print_unicode( YELLOW, dvi_ptr->string_manufacturer );
		else
			mprintf( YELLOW, CONTINUE, "Unknown printer manufacturer" );

		mprintf( YELLOW, CONTINUE, " - " );
			
		if ( dvi_ptr->string_product != NULL )
			ui_print_unicode( YELLOW, dvi_ptr->string_product );
		else
			mprintf( YELLOW, CONTINUE, "Unknown printer product" );
		
		mprintf( YELLOW, CONTINUE, "\r\n" );
	}
	
	mprintf( YELLOW, CONTINUE, "                                  choose a printer number : [ ]\r" );
	mprintf( YELLOW, CONTINUE, "                                  choose a printer number : [" );

	key		= (ui_key_wait() & 0xFF) - '0';
	
	mprintf( YELLOW, CONTINUE, "  \r                                  choose a printer number : [%d]\r\n\r\n", key );

	if ( !((unsigned int)key < num_of_prn) )
		return ( NULL );
	
	if ( pi_list[ key ]->image_buffer_ptr )
	{
		mprintf( YELLOW, CONTINUE, "Cancel printing?  \r\n" );

		if ( (devep_find_device( (pi_list[ key ])->target_device_address ))->string_product != NULL )
		{
			mprintf( YELLOW, CONTINUE, "    print process on \"" );

			ui_print_unicode( WHITE, (devep_find_device( (pi_list[ key ])->target_device_address ))->string_product );
		mprintf( YELLOW, CONTINUE, "\"\r\n" );
		}
		mprintf( YELLOW, CONTINUE, "  To continue printing, press [n] key.  \r\n" );
		mprintf( YELLOW, CONTINUE, "  To abort printing,    press other key. \r\n  " );
		mprintf( YELLOW, CONTINUE, "  >> " );
		
		if ( (ui_key_wait() & 0xFF) == 'n' )
		{	
			mprintf( YELLOW, CONTINUE, "\r\n" );
			return ( NULL );				//	if key is 'n', do nothing.
		}
	}

	mprintf( YELLOW, CONTINUE, "\r\n" );
	return ( pi_list[ key ] );
}









