/*
**  WASABI-Hot! version 1.2b   
**
**
**      -- copyright (c) 2001-2003 by Philips Japan, Ltd. -- All rights reserved --
**
**
**      ** This code has been made to check/learn                          ** 
**      **                             the ISP1362/ISP1363 functionalities **
**      ** Release 08-Dec-2003                                             **
**
**      OKANO, Akifumi
**      
**		Application Laboratory, Portable Systems
**      Semiconductors Div, Philips Japan Ltd.
**      akifumi.okano@philips.com
**      +81-3-3740-4668 
*/


/****************************************************************************/
/*	includes																*/
/****************************************************************************/

#include		<stdio.h>
#include		<string.h>
#include		<dos.h>
#include		<fcntl.h>
#include		<conio.h>
#include		<ctype.h>
#include		<time.h>

#include		"general.h"
#include		"ui.h"

#include		"_hc_core/dev_ep.h"
#include		"_hc_core/init.h"
#include		"_hc_core/isr.h"
#include		"_hc_core/port_ctl.h"
#include		"_hc_hw/hc_comm.h"
#include		"_hc_hw/hw_acces.h"

#include		"class_dr/audio/sing.h"
#include		"class_dr/hub/hub.h"
#include		"class_dr/printer/printer.h"

#include 		"_otg/otg.h"

#include		"_dc/dc_chap9.h"	// Ken
#include		"_dc/dc_main.h"
#include		"_dc/dc_hal.h"


/****************************************************************************/
/*	constants																*/
/****************************************************************************/

#define		NUMBER_OF_ASYNC_PERIODIC_PROCESS	10


/****************************************************************************/
/*	global vars																*/
/****************************************************************************/

extern unsigned		_stklen		= 0xFF00u;		//	Modify stack size

void			(*g_asynchronous_periodic_process[ NUMBER_OF_ASYNC_PERIODIC_PROCESS ])( void )	= { 
																									NULL, NULL, NULL, NULL, 
																									NULL, NULL, NULL, NULL, 
																									NULL, NULL 
																								  };
char			gp_app_name[ 13 ];
char			gp_app_path_name[ 128 ];
unsigned char	g_done					= False;


/****************************************************************************/
/*	function prototypes														*/
/****************************************************************************/

void		gene_maintenance_process( void );
void		initialize( void );
void		options_handler( int argc, char **argv );
void		give_a_name_for_this_peer( char *gp_id_string );
void		parse_music_list( char *str );
void 		setting_file_loader( void );


/****************************************************************************/
/*	function definitions													*/
/****************************************************************************/

main( int argc, char **argv )		//	Aplication starts from here!
{
	int				key;

	//
	//	Application initializations
	//

	setting_file_loader();
	options_handler( argc, argv );

	ui_start_message();


	//
	//	Hardware initializations
	//

	hwacces_io_port_access_initialize();
	init_check_1362_connection();
	
	//
	//	Hardware (1362 internal and software) initializations
	//
	
	initialize();

	gene_install_asynchronous_periodic_process( 6, gene_maintenance_process );

	//
	//	Main application loop
	//

	while ( !g_done )
	{
		ui_beat();				//	Defined in "ui.c"
		wait_ms( 0L );			//	Give a chance to execute synchronous_periodic_process (see "wait_ms()" function for more detail)
	}

	quit();

	return ( 0 );
}


void gene_maintenance_process( void )
{
	static unsigned short	is_in_execution		= False;

	if ( is_in_execution )
		return;
	else
		is_in_execution		= True;

	if ( ui_key_polling_timing() )	//	Get timing to sample keyboard hit
	{
		//	Process user command
	
		g_done	= ui_user_debug_command_handler( ui_user_debug_command_key() );
		
		//	Update screen info
		
		ui_status_monitor();
		
		// Poll the event (for topology change event)
		
		portctl_isr_flag_routine_dspatch();
	}

	if ( is_in_execution )
		is_in_execution		= False;
}


void initialize( void )		//	Initializes 1362/PC hardware and related part of software
{
	//
	//	Reset 1362
	//

	init_chip_software_reset();

	//
	//	Initialize Device, Host and OTG controller
	//
	
	DC_initialize();
	init_HC_initialize();
	OTG_initialize();

	//
	//	Install/Initialize vender specific class driver (sample implemantation)
	//		This is the driver for Griffin Technologies' PowerMate "USB volume knob (HID)"
	//

	griffin_initialize();	//	Install vender specific class driver (To use Griffin PowerMate (USB device))
}


#if 1
void wait_ms( unsigned long timeout )	//	"wait" for mili-second
{
	unsigned long			timelimit;
	unsigned char			i;
	
	timelimit	= gp_sof_counter + timeout;

	do
	{
		for ( i = 0; i < NUMBER_OF_ASYNC_PERIODIC_PROCESS; i++ )
			if ( NULL !=  g_asynchronous_periodic_process[ i ] )
				(*g_asynchronous_periodic_process[ i ])();
	}
	while ( gp_sof_counter < timelimit );
}
#else
void wait_ms( unsigned long timeout )	//	"wait" for mili-second
{
	unsigned long			timelimit;
	unsigned char			i;
	
	static unsigned long	s	=	0;


	timelimit	= gp_sof_counter + timeout;

	mprintf( GREEN, CONTINUE, " %08lX", s );

	do
	{
		for ( i = 0; i < NUMBER_OF_ASYNC_PERIODIC_PROCESS; i++ )
		{
			mprintf( GREEN, CONTINUE, " %u", i );
			
			if ( NULL !=  g_asynchronous_periodic_process[ i ] )
				(*g_asynchronous_periodic_process[ i ])();
		}
	}
	while ( gp_sof_counter < timelimit );

	mprintf( GREEN, CONTINUE, "\r %08lX", s );
	mprintf( RED, CONTINUE, "                       %08lX\r", s++ );
}
#endif


void *gene_install_asynchronous_periodic_process( unsigned char number, void (* func)( void ) )	//	This function installs the routines to execute while in "wait_ms()"
{
	/*
	 *   Slot usage table
	 *
	 *		0	Reserved for "sing"    : audio buffer maintenance routine    : sing_buffer_data_fill()
	 *		1	Reserved for "printer" : printer buffer maintenance routine  : async_read_file_and_bulk_out()
	 *		2	Reserved for DC unit   : polling func for DC                 : dc_periodic_monitor()
	 *		3	Reserved for "sing"    : Auto music continouous play ctrl    : audio_sequential_play_control()
	 *		4	Reserved for "griffin" : for temporary process               : change_blink_speed_by_mute_state()
	 *		5	Reserved for OTG unit  : OTG periodic process                : OTG_asynchronous_periodic_process()
	 *		6   Reserved for App       : Used to maintain application        : gene_maintenance_process()
	 *		7	Reserved for DC unit   : OTG remote play periph side monitor : file_acccess_monitor();
	 *		8	Reserved for "storage" : 
	 *		9	Non-reserved
	 */

	void 	*prev_func;

	if ( !(number < NUMBER_OF_ASYNC_PERIODIC_PROCESS) )
		mprintf( RED, ABORT, "error @ gene_install_asynchronous_periodic_process, installing %p on %d", func, number );
	
	prev_func	= (void *)g_asynchronous_periodic_process[ number ];
	
	g_asynchronous_periodic_process[ number ]	= func;

	//	mprintf( WHITE, CONTINUE, "gene_install_asynchronous_periodic_process %d : %p\r\n", number, func );
	
	return ( (void *)prev_func );
}





/* ********** ********** ********** ********** ********** ********** **********
	Following functions are parsers for the command-line option switches and
	setting files. 
   ********** ********** ********** ********** ********** ********** ********** */
   
void options_handler( int argc, char **argv )	//	Handler for "option command switch". It does application setting
												//		The parameter can be given as arguments from command line.
{
	static unsigned char	first_time	= 1;
	unsigned char			enable_flag;
	unsigned short			i;
	
	if ( first_time )
	{
		gp_Hc_IRQ_num			= 0xFF;
		gp_Dc_IRQ_num			= 0xFF;
		gp_ISA_base_address		= 0xFFFF;

		for ( i = 0; i < MAX_NUM_OF_AUDIO_FILES; i++ )
			*gp_sing_file_strings[ i ].file_path_and_name	= 0;
			
		first_time	= 0;
	}


	if ( argv[ 0 ] != NULL )
	{
		strcpy( gp_app_path_name, argv[ 0 ] );
		
		i	= 0;
		while ( (*(gp_app_path_name + i)	= tolower( *(gp_app_path_name + i) )) != 0 )
//		while ( *(gp_app_path_name + i)	= tolower( *(gp_app_path_name + i) ) )			<-- This is OK but TurboC default setting will warn when compile
			i++;
		
		if ( strrchr( gp_app_path_name, '\\' ) )
			strncpy( gp_app_name, strrchr( gp_app_path_name, '\\' ) + 1, 13 );
		else
			strncpy( gp_app_name, gp_app_path_name, 13 );
	}
	
	
	for ( i = 1; i < argc; i++ )
	{
		if ( *argv[ i ] == '-' )
		{
			if ( isupper( *(argv[ i ] + 1) ) )
			{
				if ( '-' == *(argv[ i ] + 2) )
					enable_flag		= Disabled;
				else
					enable_flag		= Enabled;
			}
		
			switch ( *(argv[ i ] + 1) )
			{

			/**
			 **		environment settings
			 **/
			
				case 'h' :		//	IRQ number for HC
					gp_Hc_IRQ_num	= atoi( argv[ ++i ] );
					break;
				case 'd' :		//	IRQ number for DC
					gp_Dc_IRQ_num	= atoi( argv[ ++i ] );
					break;
				case 'b' :		//	ISA base address
					sscanf( argv[ ++i ], "%X", &gp_ISA_base_address );
					break;

				case 'c' :		//	ISA base address
					if ( (*argv[ ++i ] == 'e') || (*argv[ ++i ] == 'E') )
						gp_charge_pump_external	= True;
					else
						gp_charge_pump_external	= False;	
					break;
				case 'F' :		//	for fast enumeration (no blink notice to user)
					gp_fast_enumeration		= enable_flag;
					break;
				case 'S' :		//	for silent enumeration (no beep sound for port events)
					gp_silent_enumeration	= enable_flag;
					break;
				case 'I' :		//	to skip PCI board detection
					gp_force_to_use_ISA		= enable_flag;
					break;
				case 'D' :		//	operate mode for DOS environment
					gp_is_in_DOS_environment	= enable_flag;
					break;
				case 'V' :
					gp_version_display		= enable_flag;
					break;
				case 'W' :		//	Warning message enable
					gp_warning_message		= enable_flag;
					break;
				case 'O' :		//	Old PCI PLD code
					gp_old_PCI				= enable_flag;
					break;


			/**
			 **		Audio settings
			 **/
		
				case 'a' :		//	music file selection
					strcpy( gp_sing_file_strings[ 0 ].file_path_and_name, argv[ ++i ] );
					break;
				case 'l' :		//	music LIST file selection
					strcpy( gp_sing_list_file_str, argv[ ++i ] );
					parse_music_list( gp_sing_list_file_str );
					break;
				case 'v' :		//	sound attenuation level
					audio_volume_value_set( (unsigned char)(atoi( argv[ ++i ] ) >> 1) );
					break;
					
			/**
			 **		Printer settings
			 **/
		
				case 'p' :		//	Printer file selection default path/file
					strcpy( gp_printer_default_file_path, argv[ ++i ] );
					break;
					
			/**
			 **		OTG settings
			 **/
		
				case 'E' :		//	Full enumeration at non-BusRequesting A-HOST
					gp_full_enumeration_at_non_BR_A_HOST		= enable_flag;
					break;
		
				case 'X' :		//	Modify OTG descriptor to no HNP support
					overwrite_OTG_descriptor( 1 );
					gp_emulate_peripheral_only				= enable_flag;
					break;
					
				case 'i' :		//	OTG peer name
					strncpy( gp_id_string, argv[ ++i ], ID_STRING_LENGTH );
					break;
					
					
			/**
			 **		Help settings
			 **/
		
				case '?' :
					break;
				default  :
					break;
			}
		}
	}


	/**
	 **		Set default parameters when the option switch has not been given
	 **/

	if ( *(gp_sing_file_strings[ 0 ].file_path_and_name) == 0 )
		strcpy( gp_sing_file_strings[ 0 ].file_path_and_name, AUDIO_FILENAME );
	
	if ( gp_Hc_IRQ_num == 0xFF )
		gp_Hc_IRQ_num	= DEFAULT_HC_IRQ_NUM;
	
	if ( gp_Dc_IRQ_num == 0xFF )
		gp_Dc_IRQ_num	= DEFAULT_DC_IRQ_NUM;

	if ( gp_ISA_base_address == 0xFFFF )
		gp_ISA_base_address		= ISA_BASE_ADDR;
	
	if ( *gp_id_string == 0 )
		give_a_name_for_this_peer( gp_id_string );
}


void give_a_name_for_this_peer( char *gp_id_string )
{
	char *name[ 9 ]	=	{
							"ToaPayoh",
							"Novena",
							"Newton",
							"Orchard",
							"Somerset",
							"DhobyGhaut",
							"CityHall",
							"RafflesP.",
							"MarinaBay"
						};
	
	randomize();
	strncpy( gp_id_string, name[ random( 8 ) ], ID_STRING_LENGTH );
}


unsigned long	file_size( int fh );		//	defined in printer.c

void setting_file_loader( void )	//	This is a option to set the application from file.
									//		This function call enables to do the option switch setting from file.
{
	int				file_handle;
	char			*base;
	char			*p;
	unsigned long	size;
	unsigned long	readsz;
	unsigned long	i;
	unsigned char	curr_state;
	unsigned char	prev_state	= 0;
	unsigned char	comm_state	= 0;
	unsigned short	count		= 0;
	char			**v;
	
	if ( 0 != (_dos_open( "setting.wsb", O_RDONLY, &file_handle )) )
		return;
	
	size	= file_size( file_handle );

	if ( size > 0x7FFF )
		return;
	
	if ( NULL == (base	= (char *)malloc( sizeof( char ) * size )) )
		return;
	
	_dos_read( file_handle, base, size, (unsigned *)&readsz );
	_dos_close( file_handle );

	p			= base;

	for ( i = 0; i < size; i++ )
	{
		if ( *p == '(' )
		{
			comm_state++;
		}
		else if ( *p == ')' )
		{
			*p	= ' ';
			comm_state--;
		}
		
		if ( comm_state )
			*p	= ' ';
	
		curr_state		= !isspace( *p );
		
		if ( !prev_state && curr_state )
			count++;
		
		if ( prev_state && !curr_state )
			*p	= 0;
		
		prev_state		= curr_state;
		
		p++;
	}
	
	if ( NULL == (v	= (char **)malloc( sizeof( char * ) * (count + 1) )) )
		return;
	
	v[ 0 ]		= NULL;
	count		= 1;
	p			= base;

	for ( i = 0; i < size; i++ )
	{
		if ( *p == 0 )
			curr_state		= 0;
		else
			curr_state		= !isspace( *p );
		
		if ( !prev_state && curr_state )
			v[ count++ ]	= p;
		
		prev_state		= curr_state;
		
		p++;
	}

	options_handler( count, v );
	
	free( v );
	free( base );
}


void parse_music_list( char *str )	//	Parser to decode/set music file list
{
	FILE 	*fp;
	char 	file_name[ 256 ];
	int		len;
	int		i;
	int		get_comment		= -1;

	if ( NULL == (fp	= fopen( str, "r" )) )
	{
		mprintf( LIGHTRED, CONTINUE, "error @ file open for \"-l\" option switch" );
		return;
	}
	
	
	for ( i = 0; i < MAX_NUM_OF_AUDIO_FILES; i++ )				//	Clear the list which was set previously
		*gp_sing_file_strings[ i ].file_path_and_name	= 0;	


	for ( i = 0; !((i == MAX_NUM_OF_AUDIO_FILES) && (get_comment < 0)); )
	{
		if ( NULL ==  fgets( file_name, 256, fp ))
			break;

		len		= strlen( file_name );
		len		= (len < FILE_NAME_LENGTH) ? len : (FILE_NAME_LENGTH);
		len--;
		
		if ( isspace( *( file_name + len ) ) )
			*( file_name + len )	= 0;

		if ( !len )
			continue;
		
		if ( (*file_name == '!') || (*file_name == '#') )
		{
			if ( (*(file_name + 1) == '%') && !(get_comment < 0) )
			{
				strcpy( gp_sing_file_strings[ get_comment ].file_comment, (file_name + 2) );
				get_comment		= -1;
			}
			continue;
		}
		
		get_comment		= i;
		strcpy( gp_sing_file_strings[ i++ ].file_path_and_name, file_name );
		*(gp_sing_file_strings[ get_comment ].file_comment)		= 0;
	}
	
	fclose ( fp );
}


unsigned short short_swap( unsigned short x )
{
	unsigned short	tmp;
	
	tmp		= x << 8;
	
	return ( tmp | (x >> 8) );
}


unsigned long long_swap( unsigned long x )
{
	unsigned long	tmp;
	
	tmp		 = (x & 0x000000FF) << 24;
	tmp		|= (x & 0x0000FF00) <<  8;
	tmp		|= (x & 0x00FF0000) >>  8;
	tmp		|= (x & 0xFF000000) >> 24;
	
	return ( tmp );
}





